////////////////////////////////////////////////
// DESCRIPTION:
//    GPIO Sample application
//
// Legal Notices:
//   Copyright (c) 2008, Telliam Consulting, LLC.
//   All rights reserved.
//
//   Redistribution and use in source and binary forms, with or without modification,
//   are permitted provided that the following conditions are met:
//
//   * Redistributions of source code must retain the above copyright notice, this list
//     of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above copyright notice, this
//     list of conditions and the following disclaimer in the documentation and/or other
//     materials provided with the distribution.
//   * Neither the name of Telliam Consulting nor the names of its contributors may be
//     used to endorse or promote products derived from this software without specific
//     prior written permission.
//
//   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
//   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
//   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
//   SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
//   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
//   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
//   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
//   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
//   ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
//   DAMAGE. 
//
using System;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using FusionWare.SPOT.Hardware;

namespace GPIOSample
{
    public class Program
    {
        /// <summary>Pin Number for the Button</summary>
        /// <remarks>
        /// This constant used in this sample will most likely
        /// not work on your hardware. Every hardware system
        /// has different purposes and uses for GPIO pins so
        /// you should change this to match your hardware.
        /// </remarks>
        const Cpu.Pin ButtonPin = ( Cpu.Pin )0;

        /// <summary>Pin Number for LED1</summary>
        /// <remarks>
        /// This constant used in this sample will most likely
        /// not work on your hardware. Every hardware system
        /// has different purposes and uses for GPIO pins so
        /// you should change this to match your hardware.
        /// </remarks>
        const Cpu.Pin LED1 = ( Cpu.Pin )1;

        /// <summary>Pin Number for the LED2</summary>
        /// <remarks>
        /// This constant used in this sample will most likely
        /// not work on your hardware. Every hardware system
        /// has different purposes and uses for GPIO pins so
        /// you should change this to match your hardware.
        /// </remarks>
        const Cpu.Pin LED2 = ( Cpu.Pin )2;

        public static void Main( )
        {
            // 1 minute running time for each test
            TimeSpan runTime = new TimeSpan(0, 1, 0);

            TestInputPort(runTime, ButtonPin );

            TestOutputPort(runTime, LED1 );
            TestOutputPort( runTime, LED2 );
        }

        private static void TestOutputPort(TimeSpan RunTime, Cpu.Pin Pin)
        {
            DateTime start = System.DateTime.Now;
            
            // Output port with initial state set to true (Logic level HI)
            using( OutputPort port = new OutputPort( Pin, true ) )
            {
                while( System.DateTime.Now - start < RunTime )
                {
                    port.Write( !port.Read() );
                    System.Threading.Thread.Sleep( 1000 );
                }
            }
        }

        private static void TestInputPort(TimeSpan RunTime, Cpu.Pin Pin)
        {
            DateTime start = System.DateTime.Now;
            
            // pin with no glitch filtering and resistors disabled to get raw input
            using( InputPort port = new InputPort( Pin
                                                 , false
                                                 , Port.ResistorMode.Disabled
                                                 )
                 )
            {
                while( System.DateTime.Now - start < RunTime )
                {
                    Debug.Print( "Pin is: " + port.Read().ToString() );
                    System.Threading.Thread.Sleep( 1000 );
                }
            }
        }
    }
}
