////////////////////////////////////////////////
// DESCRIPTION:
//    Micro Framework Serial port sample application
//
// Legal Notices:
//   Copyright (c) 2008, Telliam Consulting, LLC.
//   All rights reserved.
//
//   Redistribution and use in source and binary forms, with or without modification,
//   are permitted provided that the following conditions are met:
//
//   * Redistributions of source code must retain the above copyright notice, this list
//     of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above copyright notice, this
//     list of conditions and the following disclaimer in the documentation and/or other
//     materials provided with the distribution.
//   * Neither the name of Telliam Consulting nor the names of its contributors may be
//     used to endorse or promote products derived from this software without specific
//     prior written permission.
//
//   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
//   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
//   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
//   SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
//   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
//   TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
//   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
//   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
//   ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
//   DAMAGE. 
//
using System;
using System.IO;
using Microsoft.SPOT;
using System.IO.Ports;
using FusionWare.SPOT.IO;
using Microsoft.SPOT.Hardware;

namespace SerialPortSample
{
    /// <summary>Main Application class for the sample</summary>
    public class Program
    {
        /// <summary>Application Entry Point</summary>
        public static void Main()
        {
            // Simple array of bytes in ASCII: 12345678
            byte[] dummydata = { 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x0D, 0x0A };

            //Com Port to use for this program
            SerialPort serPort = new SerialPort("COM1", 2400);
            serPort.ReadTimeout = 5000;

            SerialStream ser = new SerialStream( serPort, 5000 );
            StreamWriter txtOut = new StreamWriter( ser );

            Debug.Print("Serial Port Testing...");
            txtOut.Write("Hello World!\n");
            txtOut.Write(dummydata);
            txtOut.Write("Asta la vista - I'll be back...\n");

            ReadTest(serPort);
        }

        private static void ReadTest(SerialPort SerPort)
        {
            // buffer to read data into, big enough to suck out full hardware FIFO contents
            byte[] buffer = new byte[32];
            
            // COunt of bytes read so far
            int byteCount = 0;

            // loop echoing all bytes read until 100 are read
            while(byteCount < 100)
            {
                try
                {
                    // read in the data with a 5 second time-out
                    int count = SerPort.Read( buffer, 0, buffer.Length );
                    if( count > 0 )
                    {
                        SerPort.Write( buffer, 0, count );
                        byteCount += count;
                    }
                }
                catch( System.IO.IOException ex )
                {
                    Debug.Print( "IO Exception" + ex.Message );
                }
            }
        }
    }
}
